package com.example.demo.java;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.department.model.DepartmentEntity;
import com.example.demo.news.model.NewsEntity;
import com.example.demo.news.service.NewsService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class NewsTests {
    @Autowired
    private NewsService newsService;

    private NewsEntity news;

    @BeforeEach
    void createData() {
        removeData();

        news = newsService
                .create(new NewsEntity("2024-04-13", "new1", "description1", new DepartmentEntity("ГУМ")));
        newsService.create(
                new NewsEntity("2024-05-14", "new2", "description2", new DepartmentEntity("IST")));
        newsService.create(
                new NewsEntity("2024-02-10", "new3", "description3", new DepartmentEntity("JUV")));
    }

    @AfterEach
    void removeData() {
        newsService.getAll(0L).forEach(item -> newsService.delete(item.getId()));
    }

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> newsService.get(0L));
    }

    @Test
    void createTest() {
        Assertions.assertEquals(3, newsService.getAll(0L).size());
        Assertions.assertEquals(news, newsService.get(news.getId()));
    }

    @Test
    void createNotUniqueTest() {
        final NewsEntity nonUniquedirection = new NewsEntity("2024-02-10", "new4", "description4",
                new DepartmentEntity("JUV"));
        Assertions.assertThrows(IllegalArgumentException.class, () -> newsService.create(nonUniquedirection));
    }

    @Test
    void createNullableTest() {
        final NewsEntity nullabledirection = new NewsEntity(null, null, null, null);
        Assertions.assertThrows(
                DataIntegrityViolationException.class,
                () -> newsService.create(nullabledirection));
    }

    @Test
    void updateTest() {
        final String testDate = "TESTDATE";
        final String testName = "TESTNAME";
        final String testDes = "TESTDES";
        final DepartmentEntity testDep = new DepartmentEntity("TESTDEP");
        final NewsEntity entity = newsService.get(3L);
        final String oldDate = entity.getDate();
        final String oldName = entity.getName();
        final String oldDes = entity.getDescription();
        final DepartmentEntity oldDep = entity.getDepartment();
        final NewsEntity newEntity = newsService.update(3L,
                new NewsEntity(testDate, testName, testDes, testDep));
        Assertions.assertEquals(3, newsService.getAll(0L).size());
        Assertions.assertEquals(newEntity, newsService.get(3L));

        Assertions.assertEquals(testDate, newEntity.getDate());
        Assertions.assertNotEquals(oldDate, newEntity.getDate());

        Assertions.assertEquals(testName, newEntity.getName());
        Assertions.assertNotEquals(oldName, newEntity.getName());

        Assertions.assertEquals(testDes, newEntity.getDescription());
        Assertions.assertNotEquals(oldDes, newEntity.getDescription());

        Assertions.assertEquals(testDep, newEntity.getDepartment());
        Assertions.assertNotEquals(oldDep, newEntity.getDepartment());
    }

    @Test
    void deleteTest() {
        newsService.delete(3L);
        Assertions.assertEquals(2, newsService.getAll(0L).size());
        final NewsEntity last = newsService.get(2L);
        Assertions.assertEquals(2L, last.getId());

        final NewsEntity newEntity = newsService
                .create(new NewsEntity("test1", "test2", "test3", new DepartmentEntity("testtt")));
        Assertions.assertEquals(3, newsService.getAll(0L).size());
        Assertions.assertEquals(4L, newEntity.getId());
    }
}
